import "Turbine.UI.Lotro"
import "MinstrelBuffs.MinstrelBuffWindow.MinstrelEffects";
import "MinstrelBuffs.MinstrelBuffWindow.TimerControl";
import "MinstrelBuffs.MinstrelBuffWindow.InCombatTimerControl";
import "MinstrelBuffs.MinstrelBuffWindow.DragBar";

BuffWindow = class(Turbine.UI.Window);

function BuffWindow:Constructor(settings)
	Turbine.UI.Window.Constructor(self);
	self.settings = settings;
	self.settings.ThemeChanged = function() self:Restart(); end
	self.settings.EffectWindowOnlyVisibleInCombatChanged = function() self:SetVisibility(); end
	self.WasInCombat = false;
	self.CryOfTheChorusActivated = false;
	self.MajorBalladActivated = false;
end

function BuffWindow:GetName()
	return "BuffWindow";
end

function BuffWindow:IsRunning()
	return ( self.window ~= nil );
end

function BuffWindow:ProcessKeyDown(sender, args)
    if (args.Action == KEY_ACTION_REPOSITION_UI) then
        -- lock / unlock the control for movement
		self:UiLocked(not self.locked);
    elseif (args.Action == KEY_ACTION_TOGGLE_HUD) then
        -- hide / show ui elements
        self:UiHidden(not self.hidden);
	end
end

function BuffWindow:UiLocked(locked)
    self.locked = locked;
    if (not self.locked) then
        self.dragBar = MinstrelBuffs.MinstrelBuffWindow.DragBar(self.window, "Minstrelbuffs");
        self.dragBar:SetBarOnTop(true);
        self.dragBar:SetBarVisible(true);
        self.dragBar:SetDraggable(true);
        self.SizeChanged = function(sender, args)
            self.dragBar:Layout();
        end		
    else
        if (self.dragBar ~= nil) then
            self.dragBar:SetDraggable(false);
            self.dragBar:SetTarget(nil);
            self.dragBar = nil;
        end
        self.SizeChanged = nil;
		self.settings:SetMainPos(self.window:GetPosition());
		self:SetVisibility();
    end
end
 
function BuffWindow:UiHidden(hide)
	self.hidden = hide;
	self:SetVisibility();
end

function BuffWindow:Start()
	self.locked = true;
	self.hidden = false;

	local posx = self.settings:GetMainPosX();
	local posy = self.settings:GetMainPosY();

	-- Don't create multiple windows.
	if ( self.window ~= nil ) then
		return;
	end

	-- background-Window
	--self.window = Turbine.UI.Lotro.Window();
	self.window = Turbine.UI.Window();
	if posx == nil then
		posx = 100;
	end
	if posy == nil then
		posy = 100;
	end
	local theme = self.settings:GetTheme();
	--Turbine.Shell.WriteLine(theme.name);
	self.window:SetPosition( posx, posy );
	self.window:SetSize(theme.windowWidth , theme.windowHeight );
	self.window:SetText( "Minstrel" );
	self.window:SetVisible( true );
	self.window:SetMouseVisible(false);
	self.window:SetWantsKeyEvents(true);
	self.window.KeyDown	= function(sender, args) self:ProcessKeyDown(sender, args); end
	self.window:SetBackground( theme.windowBackground );
	--self:SetBackgroundImage();
		
	self.window.Closed = function( sender, args )
		self:Stop();
	end
	
	-- Ballad background
	self.balladBackground = Turbine.UI.Control();
	self.balladBackground:SetParent(self.window);
	self.balladBackground:SetSize( theme.balladWidth, theme.balladHeight );
	self.balladBackground:SetPosition(theme.balladPosX, theme.balladPosY);
	self.balladBackground:SetVisible(true);
	self.balladBackground:SetMouseVisible(false);
	self.balladBackground:SetBackground(theme.balladBackground);
	
	-- Anthem background
	self.anthemBackground = Turbine.UI.Control();
	self.anthemBackground:SetParent(self.window);
	self.anthemBackground:SetSize( theme.anthemWidth, theme.anthemHeight );
	self.anthemBackground:SetPosition( theme.anthemPosX, theme.anthemPosY );
	self.anthemBackground:SetVisible(true);
	self.anthemBackground:SetMouseVisible(false);
	self.anthemBackground:SetBackground( theme.anthemBackground );
	
	-- Ballad effect controls
	self.balladEffectDisplays = { };
	for i = 1, 3, 1 do
		local posx = theme.balladEffectStartX + (i - 1) * theme.balladEffectSpaceX;
		self.balladEffectDisplays[i] = Turbine.UI.Lotro.EffectDisplay();
		self.balladEffectDisplays[i]:SetParent(self.window);
		self.balladEffectDisplays[i]:SetPosition( posx, theme.balladEffectY );
		self.balladEffectDisplays[i]:SetVisible(false);
		self.balladEffectDisplays[i]:SetSize(theme.balladEffectWidth, theme.balladEffectHeight);
	end
	
	-- Anthem effect controls
	self.anthemEffectDisplays = { };
	self.anthemEffectBorders = { };
	for i = 1, 6, 1 do
		local posx = theme.anthemEffectStartX + (i - 1) * theme.anthemEffectSpaceX;
		self.anthemEffectDisplays[i] = Turbine.UI.Lotro.EffectDisplay()
		self.anthemEffectDisplays[i]:SetParent(self.window);
		self.anthemEffectDisplays[i]:SetPosition( posx, theme.anthemEffectY );
		self.anthemEffectDisplays[i]:SetVisible(false);
		self.anthemEffectDisplays[i]:SetSize(theme.anthemEffectWidth, theme.anthemEffectWidth);
		
		if (theme.fixedAnthems) then
			-- Border, when the anthem is active for the next coda.
			-- Works only, when a theme with fixed anthem positions is used.
			self.anthemEffectBorders[i] = Turbine.UI.Control()
			self.anthemEffectBorders[i]:SetParent(self.window);
			self.anthemEffectBorders[i]:SetPosition( posx, theme.anthemEffectY );
			self.anthemEffectBorders[i]:SetVisible(false);
			self.anthemEffectBorders[i]:SetSize(theme.anthemEffectWidth, theme.anthemEffectWidth);
			self.anthemEffectBorders[i]:SetMouseVisible(false);
			self.anthemEffectBorders[i]:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
			self.anthemEffectBorders[i]:SetBackground(theme.anthemBorder);
		end
	end
	
	-- Anthem time controls
	self.anthemTimeDisplays = { };
	for i = 1, 6, 1 do
		local posx = theme.anthemTimeStartX + (i - 1) * theme.anthemTimeSpaceX;
		self.anthemTimeDisplays[i] = MinstrelBuffs.MinstrelBuffWindow.TimerControl()
		self.anthemTimeDisplays[i]:SetParent(self.window);
		self.anthemTimeDisplays[i]:SetPosition( posx, theme.anthemTimeY );
		self.anthemTimeDisplays[i]:SetVisible(false);
		self.anthemTimeDisplays[i]:SetTimerSize(theme.anthemTimeWidth, theme.anthemTimeHeight);
	end	

	self.inCombatTimeDisplay = MinstrelBuffs.MinstrelBuffWindow.InCombatTimerControl()
	self.inCombatTimeDisplay:SetParent(self.window);
	self.inCombatTimeDisplay:SetPosition(theme.combatTimerX, theme.combatTimerY);
	self.inCombatTimeDisplay:SetVisible(false);
	self.inCombatTimeDisplay:SetSize(theme.combatTimerWidth, theme.combatTimerHeight);
	self.inCombatTimeDisplay.CombatTimerExpired = function() self:SetVisibility(); end

	self:SetAnthemBackgroundFromStance();
	self:SetVisibility();
end

function BuffWindow:SetAnthemBackgroundFromStance()
	local theme = self.settings:GetTheme();
	local isInYellowSpec = self:IsInYellowSpec();

	if (self:GetActiveStance() == Turbine.Gameplay.Attributes.MinstrelStance.None) then
		self.window:SetBackground( theme.windowBackground );
		self.anthemBackground:SetBackground( theme.anthemBackground );
		if (isInYellowSpec and theme.anthemBackgroundYellow ~= nil) then
			self.anthemBackground:SetBackground( theme.anthemBackgroundYellow );
		end
	elseif (self:GetActiveStance() == Turbine.Gameplay.Attributes.MinstrelStance.Harmony) then
		if (theme.windowBackgroundHarmony ~= nil) then
			self.window:SetBackground( theme.windowBackgroundHarmony );
		end
		if (theme.anthemBackgroundHarmony ~= nil) then
			self.anthemBackground:SetBackground( theme.anthemBackgroundHarmony );
		end		
	elseif (self:GetActiveStance() == Turbine.Gameplay.Attributes.MinstrelStance.WarSpeech) then
		if (theme.windowBackgroundWarSpeech ~= nil) then
			self.window:SetBackground( theme.windowBackgroundWarSpeech );
		end
		if (theme.anthemBackgroundWarSpeech ~= nil) then
			self.anthemBackground:SetBackground( theme.anthemBackgroundWarSpeech );
		end		
	end
end

function BuffWindow:GetActiveStance()
	if (LocalPlayer:GetClass() == Turbine.Gameplay.Class.Minstrel and self.classAttributes == nil) then
		self.classAttributes = LocalPlayer:GetClassAttributes();
	end
	if (self.classAttributes == nil) then
		return nil;
	end
	return self.classAttributes:GetStance();
end

-- Returns true, if the minstrel is in yellow trait line.
-- Searches for "Anthem of prowess".
function BuffWindow:IsInYellowSpec()
	local skillList = LocalPlayer:GetTrainedSkills();
	local skillCount = skillList:GetCount();
	for	i = 1, skillCount, 1 do
		local skill = skillList:GetItem(i);
		local skillInfo = skill:GetSkillInfo();
		local skillName = skillInfo:GetName();
		if (self.settings:GetEffectLocalizationNames() == "fr" 
				and ValidYellowTraitSkillNamesFr[skillName] ~= nil) then
			return true;
		elseif (self.settings:GetEffectLocalizationNames() == "de" 
				and ValidYellowTraitSkillNamesDe[skillName] ~= nil) then
			return true;
		elseif (ValidYellowTraitSkillNamesEn[skillName] ~= nil) then
			return true;
		end
	end
	return false;
end

function BuffWindow:GetPosition()
	return self.window:GetPosition();
end

function BuffWindow:Stop()
	-- Free the window which will free its child controls.
	self.window = nil;
end

function BuffWindow:Restart()
	self:Stop();
	self:Start();
end

function BuffWindow:GetWindowVisible()
	return (not self.hidden) and (not self.settings:GetEffectWindowOnlyVisibleInCombat() or LocalPlayer:IsInCombat() or self:AreEffectsVisible());
end

function BuffWindow:CryOfTheChorus()
	-- In case the cry happens before the window is shown:
	if (self.inCombatTimeDisplay == nil) then
		self:Start();
	end

	self.CryOfTheChorusActivated = true;
	self:SetVisibility();
end

function BuffWindow:MajorBallad()
	-- In case the ballad happens before the window is shown:
	if (self.inCombatTimeDisplay == nil) then
		self:Start();
	end

	self.MajorBalladActivated = true;
	self:SetVisibility();
end

function BuffWindow:EnsureCombatTimerVisible()
	if (self.settings:GetShowAnthemTime()) then
		self.inCombatTimeDisplay:SetVisible(true);
	end
end

function BuffWindow:StartCombatTimer()
	self.inCombatTimeDisplay:EnteringCombat();
end

function BuffWindow:LeavingCombat()
	self.inCombatTimeDisplay:LeavingCombat();
end

function BuffWindow:BalladEnded()
	self.inCombatTimeDisplay:BalladEnded();
end

function BuffWindow:SetVisibility()
	if (self.CryOfTheChorusActivated) then
		self.CryOfTheChorusActivated = false;
		self.inCombatTimeDisplay:CryOfTheChorus();
	end
	if (self.MajorBalladActivated) then
		self.MajorBalladActivated = false;
		self.inCombatTimeDisplay:MajorBallad();
	end
	if ((LocalPlayer:IsInCombat() and not self.WasInCombat)) then
		-- Entering Combat:
		self:StartCombatTimer();
	elseif (not LocalPlayer:IsInCombat() and self.WasInCombat) then
		-- Leaving Combat:
		self:LeavingCombat();
	end
	self.WasInCombat = LocalPlayer:IsInCombat();

	if (self.window ~= nil) then
		self.window:SetVisible(self:GetWindowVisible());
		self:EnsureCombatTimerVisible();
	end
end

function BuffWindow:SetText(text)
	--self.label:SetText(text);
end

function BuffWindow:AreEffectsVisible()
	--Turbine.Shell.WriteLine(self.effectCount);
	--return self.effectCount > 0;
	for i = 1, table.getn(self.balladEffectDisplays), 1 do
		if (self.balladEffectDisplays[i]:IsVisible()) then
			return true;
		end
	end
	for i = 1, table.getn(self.anthemEffectDisplays), 1 do
		if (self.anthemEffectDisplays[i]:IsVisible()) then
			return true;
		end
	end
	if (self.inCombatTimeDisplay:GetWantsUpdates()) then
		return true;
	end
	return false;
end

function BuffWindow:GetEffectDisplay(index)
	return self.balladEffectDisplays[index];
end

function BuffWindow:GetAnthemEffectDisplay(index)
	return self.anthemEffectDisplays[index];
end

function BuffWindow:GetAnthemTimeDisplay(index)
	return self.anthemTimeDisplays[index];
end

function BuffWindow:ShowEffect(effect, index)
	local wnd = self:GetEffectDisplay(index);
	wnd:SetEffect(effect);
	wnd:SetVisible(true);
	self:SetVisibility();
end

function BuffWindow:HideEffect(index)
	local wnd = self:GetEffectDisplay(index);
	wnd:SetVisible(false);
	self:SetVisibility();
end

function BuffWindow:ShowAnthem(effect, index)
	local wnd = self:GetAnthemEffectDisplay(index);
	wnd:SetEffect(effect);
	wnd:SetVisible(true);
	
	local timeWnd = self:GetAnthemTimeDisplay(index);
	timeWnd:SetDuration(effect:GetDuration());
	timeWnd:SetStartTime(effect:GetStartTime());
	if (self.settings:GetShowAnthemTime()) then
		timeWnd:SetVisible(true);
	end
	self:SetVisibility();
end

function BuffWindow:HideAnthem(index)
	local wnd = self:GetAnthemEffectDisplay(index);
	wnd:SetVisible(false);
	local timeWnd = self:GetAnthemTimeDisplay(index);
	timeWnd:SetVisible(false);
	self:SetVisibility();
end

function BuffWindow:ShowAnthemBorder(index)
	self.anthemEffectBorders[index]:SetVisible(true);
end

function BuffWindow:HideAllAnthemBorders()
	for i = 1, table.getn(self.anthemEffectBorders), 1 do
		self.anthemEffectBorders[i]:SetVisible(false);
	end
end