import "Turbine.UI.Lotro"

SHOW_DEBUG_OPTIONS = false;

OptionWindow = class(Turbine.UI.Lotro.Window);

function OptionWindow:Constructor(settings)
	Turbine.UI.Lotro.Window.Constructor(self);

	self.settings = settings;
	self:SetPosition( 200, 200 );
	self:SetText("MinstrelBuff - Options");
	self:SetSize( 270, 400 );
	self:SetVisible(false);
	
	self:PlaceControls(self);
end

function OptionWindow:GetOptionControl()
	local control = Turbine.UI.Control();
	local height = 400;
	if (SHOW_DEBUG_OPTIONS) then
		height = 500;
	end
	control:SetSize( 270, height );
	self:PlaceControls(control);
	return control;
end

function OptionWindow:PlaceControls(parent)
	-- Combat Visibility
	self.combatVisibleCheckBox = Turbine.UI.Lotro.CheckBox();
	self.combatVisibleCheckBox:SetParent(parent);
	self.combatVisibleCheckBox:SetText("Only visible in combat");
	self.combatVisibleCheckBox:SetPosition( 20, 40 );
	self.combatVisibleCheckBox:SetSize( 230, 25 );
	self.combatVisibleCheckBox:SetChecked(self.settings:GetEffectWindowOnlyVisibleInCombat());
	self.combatVisibleCheckBox.CheckedChanged = function (sender, args) self:EffectWindowOnlyVisibleInCombatChanged(); end

	-- Theme options
	local themeLabel = Turbine.UI.Label();
	themeLabel:SetPosition( 20, 70 );
	themeLabel:SetSize( 230, 20 );
	themeLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	themeLabel:SetText("Select Theme:");
	themeLabel:SetParent(parent);
	
	local themeList = self.settings:GetThemeList();
	local themeIndex = self.settings:GetThemeIndex();
	local themeCount = table.getn(themeList);
	
	self.listBox = Turbine.UI.ListBox();
	self.listBox:SetParent( parent );
	self.listBox:SetBackColor( Turbine.UI.Color( 0.2, 0.2, 0.2 ) );
	self.listBox:SetPosition( 20, 100 );
	self.listBox:SetSize( 230, 20 * themeCount );

	for i = 1, themeCount do
		local listItem = Turbine.UI.Label();
		listItem:SetSize( 230, 20 );
		listItem:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
		listItem:SetText(themeList[i].name);
		self.listBox:AddItem( listItem );
	end
	self.listBox:SetSelectedIndex(themeIndex);
	self:ColorListBoxIndex(self.listBox);
	self.listBox.SelectedIndexChanged = function (sender, args) self:ThemeChanged(sender); end

	local y = 100 + 20 * themeCount + 20;

	-- Soliloquy Visibility
	self.soliloquyVisibleCheckBox = Turbine.UI.Lotro.CheckBox();
	self.soliloquyVisibleCheckBox:SetParent(parent);
	self.soliloquyVisibleCheckBox:SetText(GetString(_LANG.OPTIONS.USE_SOLILOQUY_TRACKER));
	self.soliloquyVisibleCheckBox:SetPosition( 20, y );
	self.soliloquyVisibleCheckBox:SetSize( 230, 25 );
	self.soliloquyVisibleCheckBox:SetChecked(self.settings:GetSolilquyWindowUsed());
	self.soliloquyVisibleCheckBox.CheckedChanged = function (sender, args) self:SolilquyWindowUsedChanged(); end
	y = y + 40;

	if (SHOW_DEBUG_OPTIONS) then
		self.debugSectionLabel = Turbine.UI.Label();
		self.debugSectionLabel:SetParent(parent);
		self.debugSectionLabel:SetSize(200, 30);
		self.debugSectionLabel:SetPosition(20, y);
		self.debugSectionLabel:SetText("Debug Options");
		y = y + 30;

		self.debugClearBalladsButton = Turbine.UI.Lotro.Button();
		self.debugClearBalladsButton:SetParent( parent );
		self.debugClearBalladsButton:SetPosition(20, y);
		self.debugClearBalladsButton:SetSize(150, 20);
		self.debugClearBalladsButton:SetText("Clear Ballads");
		self.debugClearBalladsButton.Click = function(sender, args)
			DEBUG_IGNORE_BALLAD_EFFECTS = true;
			mbMain:RefreshEffectDisplay();
			DEBUG_IGNORE_BALLAD_EFFECTS = false;
		end
		y = y + 30;

		self.debugEnterCombatButton = Turbine.UI.Lotro.Button();
		self.debugEnterCombatButton:SetParent(parent);
		self.debugEnterCombatButton:SetPosition(20, y);
		self.debugEnterCombatButton:SetSize(150, 20);
		self.debugEnterCombatButton:SetText("Enter Combat");
		self.debugEnterCombatButton.Click = function(sender, args)
			local isInCombat = true;
			mbMain:HandleCombatChanged(isInCombat);
		end
		y = y + 30;

		self.debugExitCombatButton = Turbine.UI.Lotro.Button();
		self.debugExitCombatButton:SetParent(parent);
		self.debugExitCombatButton:SetPosition(20, y);
		self.debugExitCombatButton:SetSize(150, 20);
		self.debugExitCombatButton:SetText("Exit Combat");
		self.debugExitCombatButton.Click = function(sender, args)
			local isInCombat = false;
			mbMain:HandleCombatChanged(isInCombat);
		end
		y = y + 30;

	end

end

function OptionWindow:Show()
	self:SetVisible(true);
end

function OptionWindow:ThemeChanged(sender)
	self:ColorListBoxIndex(sender);
	self.settings:SetThemeIndex(sender:GetSelectedIndex());	
end

function OptionWindow:ColorListBoxIndex(sender)
	for i = 1, sender:GetItemCount() do
		local listItem = sender:GetItem(i);
		if (i == sender:GetSelectedIndex()) then
			listItem:SetBackColor( Turbine.UI.Color( 1, 0.4, 0.4, 0.4 ) );
		else
			listItem:SetBackColor( Turbine.UI.Color( 1, 0.3, 0.3, 0.3 ) );
		end
	end
end

function OptionWindow:EffectWindowOnlyVisibleInCombatChanged()
	self.settings:SetEffectWindowOnlyVisibleInCombat(self.combatVisibleCheckBox:IsChecked());
end

function OptionWindow:SolilquyWindowUsedChanged()
	self.settings:SetSolilquyWindowUsed(self.soliloquyVisibleCheckBox:IsChecked());
end