import "MinstrelBuffs.MinstrelBuffWindow.Themes.ThemeList";
import "MinstrelBuffs.MinstrelBuffWindow.Themes.WarsteedThemeList";
import "MinstrelBuffs.MinstrelBuffWindow.SettingsEncoder"

Settings = class();

function Settings:Constructor()
	self:Load();	
end

function Settings:Load()
	local wasSettingsFileCorrect, temp = pcall(function() self.settingsTable = SettingsEncoder:DecodeTable(Turbine.PluginData.Load(Turbine.DataScope.Character, "MinstrelBuff")); end);
	
	if (self.settingsTable == nil) then
		self.settingsTable = {};
		self.settingsTable["MainPosX"] = 100;
		self.settingsTable["MainPosY"] = 100;
	end
	if (self.settingsTable["ThemeIndex"] == nil) then
		self.settingsTable["ThemeIndex"] = 1;
	end
	if (self.settingsTable["EffectWindowOnlyVisibleInCombat"] == nil) then
		self.settingsTable["EffectWindowOnlyVisibleInCombat"] = false;
	end
	if (self.settingsTable["SolilquyWindowUsed"] == nil) then
		self.settingsTable["SolilquyWindowUsed"] = false;
	end
	
	-- events
	self.ThemeChanged = nil;
	self.EffectWindowOnlyVisibleInCombatChanged = nil;
	if (not wasSettingsFileCorrect) then
		self:Save();
		Turbine.Shell.WriteLine("Minstrel Buff - Settings file corrupt. Created a new file");
	end
	--Turbine.Shell.WriteLine( "Settings:Load" );
end

function Settings:Save()
	if (self.settingsTable ~= nil) then
		Turbine.PluginData.Save(Turbine.DataScope.Character, "MinstrelBuff", SettingsEncoder:EncodeTable(self.settingsTable));
	end
end

function Settings:GetMainPosX()
	if (self.settingsTable["MainPosX"] ~= nil) then
		return self.settingsTable["MainPosX"];
	else
		return 200;
	end
end

function Settings:GetMainPosY()
	if (self.settingsTable["MainPosY"] ~= nil) then
		return self.settingsTable["MainPosY"];
	else
		return 200;
	end
end

function Settings:SetMainPos(posx, posy)
	self.settingsTable["MainPosX"] = posx;
	self.settingsTable["MainPosY"] = posy;
	self:Save();
end

function Settings:GetSoliloquyPosX()
	if (self.settingsTable["SoliloquyPosX"] ~= nil) then
		return self.settingsTable["SoliloquyPosX"];
	else
		return 200;
	end
end

function Settings:GetSoliloquyPosY()
	if (self.settingsTable["SoliloquyPosY"] ~= nil) then
		return self.settingsTable["SoliloquyPosY"];
	else
		return 200;
	end
end

function Settings:SetSoliloquyPos(posx, posy)
	self.settingsTable["SoliloquyPosX"] = posx;
	self.settingsTable["SoliloquyPosY"] = posy;
	self:Save();
end

function Settings:GetEffectWindowOnlyVisibleInCombat()
	return self.settingsTable["EffectWindowOnlyVisibleInCombat"];
end

function Settings:SetEffectWindowOnlyVisibleInCombat(value)
	self.settingsTable["EffectWindowOnlyVisibleInCombat"] = value;
	self:Save();
	if (type(self.EffectWindowOnlyVisibleInCombatChanged) == "function") then
		self.EffectWindowOnlyVisibleInCombatChanged();
	end
end

function Settings:GetSolilquyWindowUsed()
	return self.settingsTable["SolilquyWindowUsed"];
end

function Settings:SetSolilquyWindowUsed(value)
	self.settingsTable["SolilquyWindowUsed"] = value;
	self:Save();
	if (type(self.SolilquyWindowUsedChanged) == "function") then
		self.SolilquyWindowUsedChanged();
	end
end

function Settings:GetEffectLocalizationNames()
	if Turbine.Shell.IsCommand("hilfe") then
        return "de";
    elseif Turbine.Shell.IsCommand("aide") then
        return "fr";
    else
		return "en";
	end
end

-- Theme Settings
function Settings:GetFixedAnthemPositions()
	local theme = self:GetTheme();
	return theme.fixedAnthems;
end

function Settings:GetShowAnthems()
	local theme = self:GetTheme();
	return theme.showAnthems;
end

function Settings:GetShowAnthemTime()
	local theme = self:GetTheme();
	if (theme.showAnthemTime ~= nil) then
		return theme.showAnthemTime;
	else
		return true;
	end
end

function Settings:GetThemeList()
	return MinstrelBuffs.MinstrelBuffWindow.Themes.ThemeList;
end

function Settings:GetTheme()
	if (self.isOnWarsteed) then
		return MinstrelBuffs.MinstrelBuffWindow.Themes.WarsteedThemeList[self:GetWarsteedThemeIndex()];
	else
		return MinstrelBuffs.MinstrelBuffWindow.Themes.ThemeList[self:GetThemeIndex()];
	end
end

function Settings:GetThemeIndex()
	return self.settingsTable["ThemeIndex"];
end

function Settings:GetWarsteedThemeIndex()
	return 1;
end

function Settings:SetThemeIndex(value)
	self.settingsTable["ThemeIndex"] = value;
	self:Save();
	if (type(self.ThemeChanged) == "function") then
		self.ThemeChanged();
	end
end

function Settings:SetIsOnWarsteed(value)
	if (self.isOnWarsteed ~= value) then
		self.isOnWarsteed = value;
		if (type(self.ThemeChanged) == "function") then
			self.ThemeChanged();
		end
	end
end

function Settings:GetPluginVersion()
	if (self.version == nil) then
		for _,plugin in ipairs(Turbine.PluginManager.GetAvailablePlugins()) do
			if (plugin.Name == "MinstrelBuff") then
				self.version = plugin.Version;
			end
		end
		if (self.version == nil) then
		self.version = "unkown";
		end
	end
	return self.version;
end